@@ -5,8 +5,6 @@
 #' an entire map to be produced according to the graphical format specified with
 #' the `structures` argument.  
 #' 
-#' @param filename Name of plot file; default=NULL plots to screen device (low
-#' quality and likely slow)
 #' @param bbox the bounding box for the map.  A 2-by-2 matrix of 4 elements with
 #' columns of min and max values, and rows of x and y values.  If NULL, bbox is
 #' taken from the largest extent of OSM objects in osm_data.
@@ -18,36 +16,50 @@
 #' from osm_structures(), and potentially modified to alter lists of structures
 #' to be plotted, and their associated colours. Objects are overlaid on plot
 #' according to the order given in 'structures'.
-#' @param width Width of graphics device, with height calculated according to
-#' latitudinal and longitudinal proportions of 'bbox'.
 #' @param dat_prefix Prefix for data structures (default 'dat_'). Final data
 #' structures are created by appending the suffixes from osm_structures().
-#' @return list of OSM structures each as Spatial(Polygon/List)DataFrame, and
-#' appended to `osm_data` (which is NULL by default).
+#' @return List of two components: (i) List of OSM structures each as
+#' Spatial(Polygon/List)DataFrame and appended to `osm_data` (which is NULL by
+#' default), and (ii) The map as a ggplot2 object
 #' @export
 #'
 #' @section Note:
-#' If 'osm_data=NULL', then data will be downloaded, which can take some time.
-#' Progress is dumped to screen.
+#' If 'osm_data' is not given, then data will be downloaded, which can take some
+#' time.  Progress is dumped to screen.
+#'
+#' @seealso \code{\link{plot_osm_basemap}}, \code{\link{add_osm_objects}}.
+#'
+#' @examples
+#' structures <- c ('highway', 'park', 'grass')
+#' structs <- osm_structures (structures=structures, col_scheme='light')
+#' # make_osm_map returns potentially modified list of data using the provided
+#' # 'london' data:
+#' dat <- make_osm_map (osm_data=london, structures=structs)
+#' # or download data automatically using a defined bounding boox
+#' bbox <- get_bbox (c(-0.15,51.5,-0.10,51.52))
+#' \dontrun{
+#' dat <- make_osm_map (bbox=bbox, structures=structs)
+#' print (dat$map)
+#' }
 
 
-make_osm_map <- function (filename=NULL, bbox=NULL, osm_data=NULL,
-                          structures=osm_structures (), width=640,
-                          dat_prefix='dat_')
+
+make_osm_map <- function (bbox, osm_data,
+                          structures=osm_structures (), dat_prefix='dat_')
 {
-    if (is.null (bbox)) # get it from osm_data
+    if (missing (bbox)) # get it from osm_data
     {
-        if (is.null (osm_data))
+        if (missing (osm_data))
             stop ('Either bounding box or osm_data must be given')
         bbox <- matrix (c (Inf, Inf, -Inf, -Inf), nrow=2, ncol=2)
-        rownames (bbox) <- c ("x", "y")
+        rownames (bbox) <- c ('x', 'y')
         classes <- c ('SpatialLinesDataFrame', 'SpatialPolygonsDataFrame',
                       'SpatialPointsDataFrame')
         for (i in osm_data)
         {
             if (class (i) %in% classes)
             {
-                bbi <- slot (i, "bbox")
+                bbi <- slot (i, 'bbox')
                 if (bbi [1,1] < bbox [1,1]) bbox [1,1] <- bbi [1,1]
                 if (bbi [2,1] < bbox [2,1]) bbox [2,1] <- bbi [2,1]
                 if (bbi [1,2] > bbox [1,2]) bbox [1,2] <- bbi [1,2]
@@ -57,23 +69,27 @@ make_osm_map <- function (filename=NULL, bbox=NULL, osm_data=NULL,
     }
 
     sfx <- structures$suffix [1:(nrow (structures) - 1)]
-    structs_new <- which (!sapply (sfx, function (i) 
+    if (missing (osm_data))
+    {
+        structs_new <- seq (nrow (structures) - 1)
+        osm_data <- list ()
+    } else
+        structs_new <- which (!sapply (sfx, function (i) 
                            any (paste0 (dat_prefix, i) %in% names (osm_data))))
     if (length (structs_new) > 0)
     {
         structs_full <- structures
         structures <- structures [structs_new,]
 
-        cat ('Downloading and extracting OSM data for', ns, 'structures ...\n')
+        cat ('Downloading and extracting OSM data for', 
+             nrow (structures), 'structures ...\n')
         pb <- txtProgressBar (max=1, style = 3) # shows start and end positions
         t0 <- proc.time ()
         for (i in 1:nrow (structures)) {
             dat <- extract_osm_objects (key=structures$key [i],
                                         value=structures$value [i], bbox=bbox)
-            if (!is.null (dat$warn))
-                warning (paste0 (structures$structure [i], dat$warn))
             fname <- paste0 (dat_prefix, structures$suffix [i])
-            assign (fname, dat$obj)
+            assign (fname, dat)
             osm_data [[fname]] <- get (fname)
             setTxtProgressBar(pb, i / nrow (structures))
         }
@@ -85,15 +101,12 @@ make_osm_map <- function (filename=NULL, bbox=NULL, osm_data=NULL,
     ns <- nrow (structures) - 1 # last row is background
 
     bg <- structures$col [structures$structure == 'background']
-    plot_osm_basemap (bbox=bbox, filename=filename, bg=bg, width=width)
+    map <- plot_osm_basemap (bbox=bbox, bg=bg)
     for (i in seq (nrow (structures) - 1))
     {
         obji <- paste0 ('dat_', structures$suffix [i])
-        add_osm_objects (osm_data [[obji]], col=structures$cols [i])
+        map <- add_osm_objects (map, osm_data [[obji]], col=structures$cols [i])
     }
 
-    if (!is.null (filename)) 
-        junk <- dev.off (which=dev.cur())
-
-    return (osm_data)
+    list (osm_data=osm_data, map=map)
 }
